{by Daniel Berg, June 1998: enhanced version of ERombBtn from DSP}
{Added enable/disable of every single button, enhanced (nicer) arrows on the buttons}
{Added OnMouseUp and OnMouseDown event for every button}

{July 1999: VisibleButtons added to hide single Buttons}
{Jully 1999: moved     Procedure KeyDown(var Key: Word; Shift: TShiftState); Override;
    			Procedure KeyUp(var Key: Word; Shift: TShiftState); Override;
from private to public}


unit ERombBtn;

interface
uses
  WinTypes, Messages, Classes, Graphics, Controls, dialogs, sysutils;

Type
  TEnhRombButtonPressed = (erbNone, erbUp, erbLeft, erbDown, erbRight);
  TRombButtons = (rbUp, rbLeft, rbDown, rbRight);
  TButtonSet = set of TRombButtons;

  TEnhRombButtons = class(TCustomControl)
  private
    FUpColor, FDownColor: TColor;
    FOnLeftClick: TNotifyEvent;
    FOnRightClick: TNotifyEvent;
    FOnUpClick: TNotifyEvent;
    FOnDownClick: TNotifyEvent;
    FOnLeftMouseDown: TNotifyEvent;
    FOnRightMouseDown: TNotifyEvent;
    FOnUpMouseDown: TNotifyEvent;
    FOnDownMouseDown: TNotifyEvent;
    FOnLeftMouseUp: TNotifyEvent;
    FOnRightMouseUp: TNotifyEvent;
    FOnUpMouseUp: TNotifyEvent;
    FOnDownMouseUp: TNotifyEvent;
    FDrawContur: Boolean;
    FButtonSpace: Integer;
    FFullPression: Boolean;
    FWithArrows: Boolean;
    FSignalFocus: Boolean;
    FEnabledButtons: TButtonSet;
    FVisibleButtons: TButtonSet;
    Btn: Integer;
    LastKey: Word;
    BtnHalfW, BtnHalfH: Integer;
    OffSetX: Array[1..4] of Integer;
    OffSetY: Array[1..4] of Integer;
    procedure SetEnabledButtons(Value: TButtonSet);
    procedure SetVisibleButtons(Value: TButtonSet);
    Procedure WMSize(var M: TWMSize); Message wm_Size;
    Procedure WMGetDlgCode(var M: TWMGetDlgCode); Message wm_GetDlgCode;
  protected
    Procedure SetContur(Val: Boolean);
    Procedure SetButtonSpace(Val: Integer);
    Procedure SetFullPression(Val: Boolean);
    Procedure SetWithArrows(Val: Boolean);
    Procedure SetButtonDown(Val: TEnhRombButtonPressed);
    Procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); Override;
    Procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); Override;
    Function  GetButtonDown: TEnhRombButtonPressed;
    Procedure DrawArrow(B, O: Integer; Color: TColor; Clearing: Boolean);
    Procedure DrawButtonDown(B: integer);
    Procedure DrawButtonUp(B: integer);
    Procedure DoEnter; Override;
    Procedure DoExit; Override;
  public
    Constructor Create(AOwner:TComponent); Override;
    Destructor Destroy; Override;
    Procedure Paint; Override;
    Procedure KeyDown(var Key: Word; Shift: TShiftState); Override;
    Procedure KeyUp(var Key: Word; Shift: TShiftState); Override;
  published
    property ButtonUpColor: TColor Read FUpColor Write FUpColor Default clBlack;
    property ButtonDownColor: TColor Read FDownColor Write FDownColor Default clRed;
    property DrawContur: Boolean Read FDrawContur Write SetContur Default True;
    property ButtonSpace: Integer Read FButtonSpace Write SetButtonSpace Default 2;
    property FullPression: Boolean Read FFullPression Write SetFullPression Default True;
    property WithArrows: Boolean Read FWithArrows Write SetWithArrows Default True;
    property SignalFocus: Boolean Read FSignalFocus Write FSignalFocus Default True;
    property ButtonDown: TEnhRombButtonPressed Read GetButtonDown Write SetButtonDown;
    property TabStop;
    property Enabled;
    property EnabledButtons: TButtonSet read FEnabledButtons write SetEnabledButtons
	   default [rbUp, rbLeft, rbDown, rbRight];
    property VisibleButtons: TButtonSet read FVisibleButtons write SetVisibleButtons
	   default [rbUp, rbLeft, rbDown, rbRight];

    property OnLeftClick: TNotifyEvent read FOnLeftClick write FOnLeftClick;
    property OnRightClick: TNotifyEvent read FOnRightClick write FOnRightClick;
    property OnUpClick: TNotifyEvent read FOnUpClick write FOnUpClick;
    property OnDownClick: TNotifyEvent read FOnDownClick write FOnDownClick;
    property OnLeftMouseDown: TNotifyEvent read FOnLeftMouseDown write FOnLeftMouseDown;
    property OnRightMouseDown: TNotifyEvent read FOnRightMouseDown write FOnRightMouseDown;
    property OnUpMouseDown: TNotifyEvent read FOnUpMouseDown write FOnUpMouseDown;
    property OnDownMouseDown: TNotifyEvent read FOnDownMouseDown write FOnDownMouseDown;
    property OnLeftMouseUp: TNotifyEvent read FOnLeftMouseUp write FOnLeftMouseUp;
    property OnRightMouseUp: TNotifyEvent read FOnRightMouseUp write FOnRightMouseUp;
    property OnUpMouseUp: TNotifyEvent read FOnUpMouseUp write FOnUpMouseUp;
    property OnDownMouseUp: TNotifyEvent read FOnDownMouseUp write FOnDownMouseUp;
    property OnEnter;
    property OnExit;

  End;

Procedure Register;


implementation

Constructor TEnhRombButtons.Create(AOwner:TComponent);
Begin
  Inherited Create(AOwner);
  ControlStyle := ControlStyle -[csSetCaption];
  FDrawContur := True;
  FButtonSpace := 1;
  FFullPression := True;
  FWithArrows := True;
  FSignalFocus := False;
  Width := 99;
  Height := 99;
  Btn := 0;
  LastKey := 0;
  Width := 200; Height := 200;
  FEnabledButtons := [rbUp, rbLeft, rbDown, rbRight];
  FVisibleButtons := [rbUp, rbLeft, rbDown, rbRight];
  FDownColor := clRed;
End;

Destructor TEnhRombButtons.Destroy;
Begin
  Inherited Destroy;
End;



Procedure TEnhRombButtons.WMSize(var M: TWMSize);
Begin
  Inherited;
  If Width <> Height Then Height := Width;
  If FButtonSpace > (Width Div 6) Then FButtonSpace := (Width Div 6);
  If FButtonSpace < 2 Then FButtonSpace := 2;
End;

Procedure TEnhRombButtons.WMGetDlgCode(var M: TWMGetDlgCode);
Begin
  M.Result := dlgc_WantArrows;
End;

Procedure TEnhRombButtons.KeyDown(var Key: Word; Shift: TShiftState);
Begin
  Inherited KeyDown(Key, Shift);
  If Btn = 0 Then
  Begin
    Case Key of
      VK_UP   : if (rbUp in FEnabledButtons) and (rbUp in FVisibleButtons) then Btn := 1 else Key := 0;
      VK_LEFT : if (rbLeft in FEnabledButtons) and (rbLeft in FVisibleButtons) then Btn := 2 else Key := 0;
      VK_DOWN : if (rbDown in FEnabledButtons) and (rbDown in FVisibleButtons) then Btn := 3 else Key := 0;
      VK_RIGHT: if (rbRight in FEnabledButtons) and (rbRight in FVisibleButtons) then Btn := 4 else Key := 0;
    End;
    if Btn <> 0 then
    begin
     LastKey := Key;
     DrawButtonDown(Btn);
    end;
  End;
End;


procedure TEnhRombButtons.SetEnabledButtons(Value: TButtonSet);
begin
  FEnabledButtons := Value;
  DrawButtonUp(1);
  Invalidate;
end;

procedure TEnhRombButtons.SetVisibleButtons(Value: TButtonSet);
begin
  FVisibleButtons := Value;
  DrawButtonUp(1);
  Invalidate;
end;


Procedure TEnhRombButtons.KeyUp(var Key: Word; Shift: TShiftState);
Begin
  If LastKey = Key Then
  Begin
    Case Key of
      VK_UP    : begin
                  DrawButtonUp(1);
                  If Assigned(FOnUpClick)    Then FOnUpClick(Self);
                 end;
      VK_LEFT  : begin
                  DrawButtonUp(2);
                  If Assigned(FOnLeftClick)  Then FOnLeftClick(Self);
                 end;
      VK_DOWN  : begin
                  DrawButtonUp(3);
                  If Assigned(FOnDownClick)  Then FOnDownClick(Self);
                 end;
      VK_RIGHT : begin
                  DrawButtonUp(4);
                  If Assigned(FOnRightClick) Then FOnRightClick(Self);
                 end;
    End;
    Btn := 0;
  End;
  Inherited KeyUp(Key, Shift);
End;

Procedure TEnhRombButtons.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
Var
  Dx: Integer;
Begin
  Inherited MouseDown(Button, Shift, X, Y);
  SetFocus;
  If Btn = 0 Then
  Begin
    Dx := Abs(Width div 2 - X);

    If X = Y Then Exit;
    If (Y > (Width Div 2 + BtnHalfW * 2 - X)) And
        (Y < (Width Div 2*3 - BtnHalfW * 2 - X)) Then Exit;
    If (Y > (X - FButtonSpace * 2)) And
        (Y < (X + FButtonSpace * 2)) Then Exit;

    If X < Width Div 2 Then
    Begin
      If y < Height div 2 - dx Then
      Begin
        If y > dx Then Btn := 1;
      End
      else If (y > Height div 2 - dx) and (y < Height div 2 + dx) Then
      Begin
        If (y > dx) and (y < Height - dx) Then Btn := 2;
      End
      else
      Begin
        If y < Height - dx Then Btn := 3;
      End;
    End
    else
    Begin
      If y < Height div 2 - dx Then
      Begin
        If y > dx Then Btn := 1;
      End
      else If (y > Height div 2 - dx) and (y < Height div 2 + dx) Then
      Begin
        If (y > dx) and (y < Height - dx) Then Btn := 4;
      End
      else
      Begin
        If y < Height - dx Then Btn := 3;
      End;
    End;

    Case Btn of
      1: begin
           if (rbUp in FEnabledButtons) and (rbUp in FVisibleButtons) then
              begin
              DrawButtonDown(Btn);
              If Assigned(FOnUpMouseDown)    Then FOnUpMouseDown(Self);
              end;
         end;
      2: begin
           if (rbLeft in FEnabledButtons) and (rbLeft in FVisibleButtons) then
              begin
              DrawButtonDown(Btn);
              If Assigned(FOnLeftMouseDown)  Then FOnLeftMouseDown(Self);
              end;
         end;
      3: begin
           if (rbDown in FEnabledButtons) and (rbDown in FVisibleButtons) then
              begin
              DrawButtonDown(Btn);
              If Assigned(FOnDownMouseDown)  Then FOnDownMouseDown(Self);
              end;
         end;
      4: begin
           if (rbRight in FEnabledButtons) and (rbRight in FVisibleButtons) then
              begin
              DrawButtonDown(Btn);
              If Assigned(FOnRightMouseDown) Then FOnRightMouseDown(Self);
              end;
         end;
    End;

  End;
End;

Procedure TEnhRombButtons.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
Begin

  If Btn > 0 Then DrawButtonUp(Btn);
  Case Btn of
      1: if rbUp in FEnabledButtons then
         begin
          If Assigned(FOnUpClick)    Then FOnUpClick(Self);
          If Assigned(FOnUpMouseUp)    Then FOnUpMouseUp(Self);
         end;
      2: if rbLeft in FEnabledButtons then
         begin
           If Assigned(FOnLeftClick)  Then FOnLeftClick(Self);
           If Assigned(FOnLeftMouseUp)  Then FOnLeftMouseUp(Self);
         end;
      3: if rbDown in FEnabledButtons then
         begin
           If Assigned(FOnDownClick)  Then FOnDownClick(Self);
           If Assigned(FOnDownMouseUp)  Then FOnDownMouseUp(Self);
         end;
      4: if rbRight in FEnabledButtons then
         begin
           If Assigned(FOnRightClick)  Then FOnRightClick(Self);
           If Assigned(FOnRightMouseUp) Then FOnRightMouseUp(Self);
         end;
    End;
  Inherited MouseUp (Button, Shift, X, Y);

  Btn := 0;
  End;

Procedure TEnhRombButtons.SetContur(Val:Boolean);
Begin
  If Val <> FDrawContur Then
  Begin
    FDrawContur := Val;
    Invalidate;
  End;
End;

Procedure TEnhRombButtons.SetButtonSpace(Val: Integer);
Begin
  If Val <> FButtonSpace Then
  Begin
    If Val > (Width Div 6) Then Val := (Width Div 6);
    If Val < 2 Then Val := 1;
    FButtonSpace := Val;
    Invalidate;
  End;
End;

Procedure TEnhRombButtons.SetFullPression(Val: Boolean);
Begin
  If Val <> FFullPression Then
  Begin
    FFullPression := Val;
    Invalidate;
  End;
End;

Procedure TEnhRombButtons.SetWithArrows(Val: Boolean);
Begin
  If Val <> FWithArrows Then
  Begin
    FWithArrows := Val;
    Invalidate;
  End;
End;

Procedure TEnhRombButtons.SetButtonDown(Val: TEnhRombButtonPressed);
Begin
  If Btn > 0 Then DrawButtonUp(Btn);
  Btn := Ord(Val);
  If Btn > 0 Then DrawButtonDown(Btn);
End;

Function TEnhRombButtons.GetButtonDown: TEnhRombButtonPressed;
Begin
  Result := TEnhRombButtonPressed(Btn);
End;

Procedure TEnhRombButtons.DrawArrow(B, O: Integer; Color: TColor; Clearing: Boolean);
Begin
 With Canvas do
 begin
   Pen.Color := color;
   Brush.Color := color;
   Pen.Width := 1;

  case B of
    1: begin
         Polygon([Point(BtnHalfW + OffSetX[B]+O, BtnHalfH Div 2 + OffSetY[B]),
            Point((BtnHalfW  + OffSetX[B]+O) - Trunc((BtnHalfW  + OffSetX[B]+O)/8), BtnHalfH + OffSetY[B]),
            Point((BtnHalfW  + OffSetX[B]+O) + Trunc((BtnHalfW  + OffSetX[B]+O)/8), BtnHalfH + OffSetY[B])]);

         if not Clearing then
         begin
          Pen.Color := clBtnShadow;
          Brush.Color := clBtnShadow;
         end;
         MoveTo(BtnHalfW + OffSetX[B]+O, BtnHalfH Div 2 + OffSetY[B]);
         LineTo((BtnHalfW  + OffSetX[B]+O) - Trunc((BtnHalfW  + OffSetX[B]+O)/8), BtnHalfH + OffSetY[B]);
         if not Clearing then
         begin
          Pen.Color := Clwhite;
          Brush.Color := ClWhite;
         end;
         LineTo((BtnHalfW  + OffSetX[B]+O) + Trunc((BtnHalfW  + OffSetX[B]+O)/8), BtnHalfH + OffSetY[B]);
         LineTo(BtnHalfW + OffSetX[B]+O, BtnHalfH Div 2 + OffSetY[B]);
       end;

    2: begin
         Polygon([Point(BtnHalfW Div 2 + OffSetX[B]+O, BtnHalfH + OffSetY[B]),
            Point(BtnHalfW + OffSetX[B]+O, BtnHalfH + OffSetY[B] - Trunc((BtnHalfH + OffSetY[B])/8)),
            Point(BtnHalfW + OffSetX[B]+O, BtnHalfH + OffSetY[B] + Trunc((BtnHalfH + OffSetY[B])/8))]);

         if not Clearing then
         begin
          Pen.Color := clBtnShadow;
          Brush.Color := clBtnShadow;
         end;
         MoveTo(BtnHalfW Div 2 + OffSetX[B]+O, BtnHalfH + OffSetY[B]);
         LineTo(BtnHalfW + OffSetX[B]+O, BtnHalfH + OffSetY[B] - Trunc((BtnHalfH + OffSetY[B])/8));
         if not Clearing then
         begin
          Pen.Color := Clwhite;
          Brush.Color := ClWhite;
         end;
         LineTo(BtnHalfW + OffSetX[B]+O, BtnHalfH + OffSetY[B] + Trunc((BtnHalfH + OffSetY[B])/8));
         LineTo(BtnHalfW Div 2 + OffSetX[B]+O, BtnHalfH + OffSetY[B]);
    end;

    3: begin
         Polygon([Point(BtnHalfW + OffSetX[B]+O, BtnHalfH Div 2*3 + OffSetY[B]),
            Point((BtnHalfW  + OffSetX[B]+O) - Trunc((BtnHalfW  + OffSetX[B]+O)/8), BtnHalfH + OffSetY[B]),
            Point((BtnHalfW  + OffSetX[B]+O) + Trunc((BtnHalfW  + OffSetX[B]+O)/8), BtnHalfH + OffSetY[B])]);

         if not Clearing then
         begin
          Pen.Color := clBtnShadow;
          Brush.Color := clBtnShadow;
         end;
         MoveTo(BtnHalfW + OffSetX[B]+O, BtnHalfH Div 2*3 + OffSetY[B]);
         LineTo((BtnHalfW  + OffSetX[B]+O) - Trunc((BtnHalfW  + OffSetX[B]+O)/8), BtnHalfH + OffSetY[B]);
         LineTo( (BtnHalfW  + OffSetX[B]+O) + Trunc((BtnHalfW  + OffSetX[B]+O)/8), BtnHalfH + OffSetY[B]);
         if not Clearing then
         begin
          Pen.Color := Clwhite;
          Brush.Color := ClWhite;
         end;
         LineTo(BtnHalfW + OffSetX[B]+O, BtnHalfH Div 2*3 + OffSetY[B]);
       end;

    4: begin
         Polygon([Point(BtnHalfW Div 2*3 + OffSetX[B]+O, BtnHalfH + OffSetY[B]),
            Point(BtnHalfW + OffSetX[B]+O, BtnHalfH + OffSetY[B] - Trunc((BtnHalfH + OffSetY[B])/8)),
            Point(BtnHalfW + OffSetX[B]+O, BtnHalfH + OffSetY[B] + Trunc((BtnHalfH + OffSetY[B])/8))]);

         if not Clearing then
         begin
          Pen.Color := clBtnShadow;
          Brush.Color := clBtnShadow;
         end;
         MoveTo(BtnHalfW Div 2*3 + OffSetX[B]+O, BtnHalfH + OffSetY[B]);
         LineTo(BtnHalfW + OffSetX[B]+O, BtnHalfH + OffSetY[B] - Trunc((BtnHalfH + OffSetY[B])/8));
         LineTo(BtnHalfW + OffSetX[B]+O, BtnHalfH + OffSetY[B] + Trunc((BtnHalfH + OffSetY[B])/8));
         if not Clearing then
         begin
          Pen.Color := Clwhite;
          Brush.Color := ClWhite;
         end;
         LineTo(BtnHalfW Div 2*3 + OffSetX[B]+O, BtnHalfH + OffSetY[B]);
       end;

  end;
 end;
end;


Procedure TEnhRombButtons.DrawButtonDown(B: Integer);
var i: byte;
    RBtn: TRombButtons;

Begin
  With Canvas do
  Begin
    If FWithArrows Then
    Begin
      DrawArrow(B, 0, clBtnFace, True);
      DrawArrow(B, 1, FDownColor,False);
    End;
    If FFullPression Then Pen.Width := 2
    Else Pen.Width := 1;

    Pen.Color:=clBtnShadow;
    MoveTo(BtnHalfW + OffSetX[B], OffSetY[B]);
    LineTo(OffSetX[B], BtnHalfH + OffSetY[B]);
    LineTo(BtnHalfW + OffSetX[B], BtnHalfH*2 + OffSetY[B]);

    Pen.Color:=clBtnHighlight;
    MoveTo(BtnHalfW + OffSetX[B], OffSetY[B]);
    LineTo(BtnHalfW*2 + OffSetX[B], BtnHalfH + OffSetY[B]);
    LineTo(BtnHalfW + OffSetX[B], BtnHalfH*2 + OffSetY[B]);

  If DrawContur then
    Begin
      Pen.Width := 1;
      Pen.Color := clBlack;
      MoveTo(BtnHalfW + OffSetX[B], OffSetY[B]-1);
      LineTo(OffSetX[B]-1, BtnHalfH + OffSetY[B]);
      LineTo(BtnHalfW + OffSetX[B], BtnHalfH*2 + OffSetY[B]+1);
      LineTo(BtnHalfW*2 + OffSetX[B]+1, BtnHalfH + OffSetY[B]);
      LineTo(BtnHalfW + OffSetX[B], OffSetY[B]-1);
    End;
  End;
 End;

Procedure TEnhRombButtons.DrawButtonUp(B: Integer);
var i: byte;
    RBtn: TRombButtons;

Begin
if TRombButtons(B-1) in FVisibleButtons then
  begin
  With Canvas do
  Begin
    If FWithArrows Then
       if TRombButtons(B-1) in FEnabledButtons then DrawArrow(B, 0, FUpColor, False) else DrawArrow(B, 0, clBtnFace, False);

    Pen.Width := 2;
    Pen.Color:=clBtnHighlight;
    MoveTo(BtnHalfW + OffSetX[B], OffSetY[B]);
    LineTo(OffSetX[B], BtnHalfH + OffSetY[B]);
    LineTo(BtnHalfW + OffSetX[B], BtnHalfH*2 + OffSetY[B]);

    Pen.Color:=clBtnShadow;
    MoveTo(BtnHalfW + OffSetX[B], OffSetY[B]);
    LineTo(BtnHalfW*2 + OffSetX[B], BtnHalfH + OffSetY[B]);
    LineTo(BtnHalfW + OffSetX[B], BtnHalfH*2 + OffSetY[B]);

    If DrawContur then
    Begin
      Pen.Width := 1;
      Pen.Color := clBlack;

      MoveTo(BtnHalfW + OffSetX[B], OffSetY[B]-1);
      LineTo(OffSetX[B]-1, BtnHalfH + OffSetY[B]);
      LineTo(BtnHalfW + OffSetX[B], BtnHalfH*2 + OffSetY[B]+1);
      LineTo(BtnHalfW*2 + OffSetX[B]+1, BtnHalfH + OffSetY[B]);
      LineTo(BtnHalfW + OffSetX[B], OffSetY[B]-1);
    End;
  End;
 end;
End;

Procedure TEnhRombButtons.Paint;
var i: byte;
Begin
  Inherited Paint;
  BtnHalfW := Width Div 4 - FButtonSpace;
  BtnHalfH := Height Div 4 - FButtonSpace;
  OffSetX[1] := Width Div 4 + FButtonSpace - 2;
  OffSetX[2] := 0;
  OffSetX[3] := Width Div 4 + FButtonSpace - 2;
  OffSetX[4] := Width Div 2 + (FButtonSpace-2) * 2;
  OffSetY[1] := 0;
  OffSetY[2] := Height Div 4 + FButtonSpace-2;
  OffSetY[3] := Height Div 2 + (FButtonSpace-2) * 2;
  OffSetY[4] := Height Div 4 + FButtonSpace-2;
  for i := 1 to 4 do DrawButtonUp(i);
End;

Procedure TEnhRombButtons.DoEnter;
Begin
  Inherited DoEnter;
  If FSignalFocus Then
  With Canvas do
  Begin
    Pen.Color := clBlack;
    Pen.Width := 1;
    MoveTo(0, 0);
    LineTo(Width-3, 0);
    LineTo(Width-3, Height-3);
    LineTo(0, Height-3);
    LineTo(0, 0);
  End;
End;

Procedure TEnhRombButtons.DoExit;
Begin
  Inherited DoExit;
  If FSignalFocus Then
  With Canvas do
  Begin
    Pen.Color := clBtnFace;
    Pen.Width := 1;
    MoveTo(0, 0);
    LineTo(Width-3, 0);
    LineTo(Width-3, Height-3);
    LineTo(0, Height-3);
    LineTo(0, 0);
  End;
End;

Procedure Register;
Begin
  RegisterComponents('Examples', [TEnhRombButtons]);
End;

End.
